<?php
/*
 * last modified---
 * 	03-20-12/kbw/ add use of pkcs5_pad()
 * 	07-14-11/kbw/ cosmetic
 * 	07-19-10/kbw/ support multiple possible Issuers
 * 	08-26-09/kbw/ new
 *
 * purpose---
 * 	provide functionality to pass a website order's information to a voucher
 * 	safe client which the buyer will use to make payment
 */

// This package requires the XMPPHP package, which you may obtain from:
// http://code.google.com/p/xmpphp/
include 'XMPPHP/XMPP.php';
// NB: also, your php5 install will need to have the mcrypt package available

/**
 * Voucher SCI main class
 * @author Kevin Wilkerson <kbw@rayservers.net>
 * @copyright 2009-2011 Voucher-Safe.org Open Source Project
 */
class VoucherSCI {
	/**
	 * @var string
	 * the URL of the seller's website where order is being placed
	 */
	protected $website;

	/**
	 * @var string
	 * the XMPP server where the seller's bot can log in
	 */
	protected $botXMPPhost;

	/**
	 * @var integer
	 * the port number of the bot's XMPP server
	 */
	protected $botXMPPport;

	/**
	 * @var string
	 * the username at which the bot logs into the XMPP server
	 */
	protected $botUser;

	/**
	 * @var string
	 * the bot user's XMPP password
	 */
	protected $botPass;

	/**
	 * @var string
	 * the bot's XMPP resource string
	 */
	protected $botResource;

	/**
	 * @var object
	 * the XMPPHP connection object
	 */
	protected $botXMPPconn;

	/**
	 * @var string
	 * the seller's voucher safe number
	 */
	protected $sellerVS;

	/**
	 * constructor
	 *
	 * @param string	$site			seller website URL
	 * @param string	$host			bot XMPP login host
	 * @param integer	$port			bot XMPP login port (default 5222)
	 * @param string	$user			bot XMPP login username
	 * @param string	$password	bot XMPP login password
	 * @param string	$resource	bot XMPP login resource (optional)
	 */
	public function __construct($site,
										 $host,
										 $port = 5222,
										 $user,
										 $password,
										 $resource = "xmpphp")
	{
		$this->website = $site;
		$this->botXMPPhost = $host;
		$this->botXMPPport = $port;
		$this->botUser = $user;
		$this->botPass = $password;
		$this->botResource = $resource;
	}

	/**
	 * set the seller's VS number
	 * @param string	$VSnum
	 */
	public function setSellerVS($VSnum) {
		$this->sellerVS = $VSnum;
	}

	/**
	 * pass the order information to the buyer's VSC at their JID
	 * @param string $buyerJID the buyer's JID (at which they must be logged in)
	 * @param string $subject the text to use as <subject/> in message to buyer
	 * @param string $amount the total payment seller is requesting
	 * @param string $currCode the currency code in which amount is denominated
	 * @param string $issuer the voucher Issuer requested by seller
	 * @param array $orderInfo mapping of name/value pairs describing order
	 * @param string $secret the encryption key to use to encrypt message <body/>
	 */
	public function sendOrderInfo($buyerJID,
											$subject,
											$amount,
											$currCode,
											$issuer,
											$orderInfo,
											$secret)
	{
		// make connection if we don't have one or have a dead one
		$freshConn = false;
		if (is_null($botXMPPconn)) {
			$freshConn = true;
			$botXMPPconn = new XMPPHP_XMPP($this->botXMPPhost,
													 $this->botXMPPport,
													 $this->botUser,
													 $this->botPass,
													 $this->botResource,
													 '',
													 false,
													 XMPPHP_Log::LEVEL_INFO);
			$botXMPPconn->useEncryption(true);		// mandate TLS
		}
		if ($freshConn || $botXMPPconn->isDisconnected()) {
			try {
				$botXMPPconn->connect(30, true, true);
				$botXMPPconn->processUntil('session_start');
				$botXMPPconn->presence();
			}
			catch(XMPPHP_Exception $xe) {
				die($xe->getMessage());
			}
		}

		// build the message constituents other than the payload
		$to = htmlspecialchars($buyerJID);
		$body = "Your order details; decrypt in Payment form with code from "
					. "{$this->website}";

		// build the <web_vsc/> extension element
		$payload = "<web_vsc xmlns='jabber:message:voucher-safe#web-vsc'>";
		$payload .= "<info>";
		$contents .= "<opcode>NFY_order_info</opcode>";
		$contents .= "<website>{$this->website}</website>";
		$contents .= "<VSnumber>{$this->sellerVS}</VSnumber>";
		$contents .= "<amount>{$amount}</amount>";
		$contents .= "<currency>{$currCode}</currency>";
		$contents .= "<issuer>{$issuer}</issuer>";
		$keys = array_keys($orderInfo);
		$keyList = "";
		$firstKey = true;
		foreach ($keys as $key) {
			if (!$firstKey) {
				$keyList .= ":--:";
			}
			$keyList .= $key;
			$firstKey = false;
		}
		$contents .= "<baggageFields>{$keyList}</baggageFields>";
		foreach ($orderInfo as $field => $value) {
			$contents .= "<{$field}>{$value}</{$field}>";
		}

		// Encrypt the contents with the secret.  Because triple-DES uses a 3x8 or
		// 24-byte key, we'll append extra null bytes onto the secret as needed.

		$key = str_pad($secret, 24, "\x00", STR_PAD_RIGHT);
		$td = mcrypt_module_open(MCRYPT_3DES, '', MCRYPT_MODE_ECB, '');
		// the initialization vector is ignored in ECB mode, but we still need one
		$iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
		mcrypt_generic_init($td, $key, $iv);
		// we have to supply PKCS#5 padding manually, because mcrypt doesn't do it
		$paddedContents = $this->pkcs5_pad($contents);
		$encrypted_data = mcrypt_generic($td, $paddedContents);
		mcrypt_generic_deinit($td);
		mcrypt_module_close($td);
		$encContents = base64_encode($encrypted_data);
		$payload .= $encContents;
		$payload .= "</info>";
		$payload .= "</web_vsc>";

		// send the message
		$botXMPPconn->message($buyerJID, $body, 'normal', $subject, $payload);
	}

	/**
	 * provide PKCS#5 padding equivalent
	 * @param $text the text to be padded
	 * @return string the padded text
	 */
	public function pkcs5_pad($text) {
		$blocksize = mcrypt_get_block_size('tripledes', 'ecb');		// 8
		$pad = $blocksize - (strlen($text) % $blocksize);
		return $text . str_repeat(chr($pad), $pad);
	}

	/**
	 * destructor
	 */
	public function __destruct() {
		$botXMPPconn = NULL;
	}

	/**
	 * pick an encryption secret
	 * @param $digits number of digits to use
	 * @return string the secret chosen (a hex number)
	 */
	public function pickSecret($digits) {
		// compute MD5 hash of random number, and use the indicated # of digits
		$rand = mt_rand();
		$hash = md5($rand);
		return substr($hash, 0, $digits);
	}
}
?>
